@@ -12,6 +12,7 @@ import numpy as np
 import copy
 from tabulate import tabulate
 from .constants import *
+import math
 
 class MCDA:
     """
@@ -173,6 +174,7 @@ class MCDA:
     def set_weights_by_entropy(self, normalization_method_for_entropy=LinearSum_):
         """
         Set weights by entropy for MCDM problems and save it into attribute: weights.
+        IMPORTANT: signals must have to be set before weights
 
         :param normalization_method_for_entropy: (default=LinearSum_) 0 results in the same weight for all criteria,
         :type normalization_method_for_entropy: some value defined in NORMALIZATION_METHODS constant
@@ -180,6 +182,11 @@ class MCDA:
         :returns: no value
         :rtype: none
         """
+
+        # check if signal are defined before weigths by entropy.   They are required for normalization
+        if self.signals == []:
+            raise ReferenceError("Signals must have to be set before weights to use entropy")
+
         # Save current norm method
         current_norm_method = self.normalization_method
 
@@ -188,20 +195,23 @@ class MCDA:
 
         self.__normalize()
         x, y = self.df_normalized.iloc[:,1:].shape
-        entropies = np.empty(y)
 
-        for i, col in enumerate(self.df_normalized.iloc[:,1:].T):
-            if np.any(col == 0):
-                entropies[i] = 0
-            else:
-                entropies[i] = -np.sum(col * np.log(col))
-        entropies = entropies / np.log(x)
+        h = 1 / math.log(x)
 
-        result = 1 - entropies
+        entropies = -h * (self.df_normalized.iloc[:, 1:] * np.log(self.df_normalized.iloc[:, 1:])).sum(axis=0)
+
+        distances = pd.DataFrame(1 - entropies)
+        distances_sum = distances.iloc[:,:].sum(axis=0)
+
+        # Weights vector
+        W = distances/distances_sum.values
+
+        w = []
+        for i in W.values.tolist():
+            w.append(i[0])
         
         # set entropy weights
-        self.weights = (result/np.sum(result)).tolist()
-
+        self.weights = w
 
         #back current nor method
         self.normalization_method = current_norm_method
